@@ -95,8 +95,9 @@ built-in instruments to play a sample from disk/memory, offline rendering, etc.
         # Automate the cutoff freq. of the filter
         filt.automatep('kcutoff', [0, 2000, dur*0.8, 500, dur, 6000], delay=start)
 """
+import sys
 from .dependencies import checkDependencies
-checkDependencies()
+checkDependencies(force=False, tryfix=True)
 
 from .engine import *
 from .config import config
@@ -105,6 +106,6 @@ from .instr import Instr
 from .offline import Renderer
 from . import csoundlib
 from .csoundlib import (dumpAudioDevices, dumpAudioBackends,
-                        getAudioBackend, availableAudioBackends)
+                        getAudioBackend, audioBackends)
 
 
