@@ -10,7 +10,8 @@ from typing import Dict, Optional as Opt, List, Union as U, Sequence as Seq, Tup
 class Instr:
     """
     An Instr is a template used to schedule a concrete instrument
-    at a Session or a Renderer. It must be registered to be used.
+    at a :class:`~csoundengine.session.Session` or a :class:`~csoundengine.offline.Renderer`.
+    It must be registered to be used.
 
     Args:
         name: the name of the instrument
@@ -31,28 +32,28 @@ class Instr:
             if True, the associated table is freed when the note is finished
         doc: some documentation describing what this instr does
 
+
     Attributes:
         name: the name of this Instr
         body: the body of the instr (the text inside instr xxx/endin)
-        args: a dict like `{'ibus': 1, 'kfreq': 440}`, defining pfields and
+        args: a dict like ``{'ibus': 1, 'kfreq': 440}``, defining pfields and
             their default values. The mapping between pfield name and index
             is done by order, starting with p5.
         init: any global code needed
         tabledef: similar to args, but not using pfields but a table. In this
             case all variables are k-type and do not need the "k" prefix
         numchans: currently not used
-        preschedCallback: a callback called whenever an instance of this Instr
-            is scheduled
         freetable: if True, the Instr generates code to free the param table
         doc: some text documenting the use/purpose of this Instr
 
+
     Example
     =======
 
-    .. code::
+    .. code-block:: python
 
-        s = Engine().getSession()
-        Intr('sine', r'''
+        s = Engine().session()
+        Instr('sine', r'''
             kfreq = p5
             kamp = p6
             a0 = oscili:a(kamp, kfreq)
@@ -63,15 +64,16 @@ class Instr:
 
     An Instr can define default values for any of its p-fields:
 
-    .. code::
+    .. code-block:: python
 
-        s = Engine().getSession()
-        Intr('sine', args={'kamp': 0.1, 'kfreq': 1000, body=r'''
+        s = Engine().session()
+        Instr('sine', r'''
             kamp = p5
             kfreq = p6
             a0 = oscili:a(kamp, kfreq)
             outch 1, a0
-        ''').register(s)
+        ''', args={'kamp': 0.1, 'kfreq': 1000}
+        ).register(s)
         # We schedule an event of sine, kamp will take the default (0.1)
         synth = s.sched('sine', kfreq=440)
         synth.stop()
@@ -80,8 +82,8 @@ class Instr:
 
     .. code::
 
-        s = Engine().getSession()
-        Intr('sine', r'''
+        s = Engine().session()
+        Instr('sine', r'''
             |kamp=0.1, kfreq=1000|
             a0 = oscili:a(kamp, kfreq)
             outch 1, a0
@@ -93,8 +95,8 @@ class Instr:
 
     .. code-block:: python
 
-        s = Engine().getSession()
-        Intr('sine', r'''
+        s = Engine().session()
+        Instr('sine', r'''
             a0 = oscili:a(kamp, kfreq)
             outch 1, a0
         ''', tabledef=dict(amp=0.1, freq=1000
@@ -102,6 +104,18 @@ class Instr:
         synth = s.sched('sine', tabargs=dict(freq=440))
         synth.stop()
 
+
+    An inline syntax exists also for tables:
+
+    .. code::
+
+        Intr('sine', r'''
+            {amp=0.1, freq=1000}
+            a0 = oscili:a(kamp, kfreq)
+            outch 1, a0
+        ''')
+
+
     This will create a table and fill it will the given/default values,
     and generate code to read from the table and free the table after
     the event is done. Call :meth:`~csoundengine.instr.Instr.dump` to see
@@ -122,24 +136,49 @@ class Instr:
         a0 = oscili:a(kamp, kfreq)
         outch 1, a0
 
-    An inline syntax exists also for tables:
+    An Instr can also be used to define instruments for offline rendering (see
+    :class:`~csoundengine.offline.Renderer`)
 
-    .. code::
+    .. code-block:: python
 
-        Intr('sine', r'''
-            {amp=0.1, freq=1000}
-            a0 = oscili:a(kamp, kfreq)
-            outch 1, a0
-        ''')
+        from csoundengine import *
+        renderer = Renderer(sr=44100, nchnls=2)
+
+        instrs = [
+            Instr('saw', r'''
+              kmidi = p5
+              outch 1, oscili:a(0.1, mtof:k(kmidi))
+            '''),
+            Instr('sine', r'''
+              |kamp=0.1, kmidi=60|
+              asig oscili kamp, mtof:k(kmidi)
+              asig *= linsegr:a(0, 0.1, 1, 0.1, 0)
+              outch 1, asig
+            ''')
+        ]
+
+        for instr in instrs:
+            instr.register(renderer)
+
+        score = [('saw', 0,   2, 60),
+                 ('sine', 1.5, 4, 67),
+                 ('saw', 1.5, 4, 67.1)]
+
+        events = [renderer.sched(ev[0], delay=ev[1], dur=ev[2], pargs=ev[3:])
+                  for ev in score]
+
+        # offline events can be modified just like real-time events
+        renderer.automatep(events[0], 'kmidi', pairs=[0, 60, 2, 59])
+        renderer.setp(events[1], 3, 'kmidi', 67.2)
+        renderer.render("out.wav")
 
     """
     __slots__ = (
-    'body', 'name', 'args', 'init', '_tableDefaultValues', '_tableNameToIndex', 'tabledef',
-    'numchans', 'mustFreeTable', 'doc',
-    '_numpargs', '_recproc', '_check', '_preschedCallback',
-    'pargsIndexToName',
-    'pargsNameToIndex',
-    'pargsDefaultValues')
+        'body', 'name', 'args', 'init', '_tableDefaultValues', '_tableNameToIndex',
+        'tabledef', 'numchans', 'mustFreeTable', 'doc',
+        'pargsIndexToName', 'pargsNameToIndex', 'pargsDefaultValues',
+        '_numpargs', '_recproc', '_check', '_preschedCallback',
+    )
 
     def __init__(self,
                  name: str,
@@ -200,7 +239,7 @@ class Instr:
         self.init = init if init else None
         self.numchans = numchans
         self.doc = doc
-        self._numpargs = None
+        self._numpargs: Opt[int] = None
         self._recproc = None
         self._check = config['check_pargs']
         self._preschedCallback = preschedCallback
@@ -229,7 +268,10 @@ class Instr:
             return ", ".join(
                     f"p{i}: {pname}" for i, pname in sorted(pargs.items()) if i != 4)
 
-    def dump(self):
+    def dump(self) -> str:
+        """
+        Returns a string with the generated code of this Instr
+        """
         header = f"Instr(name='{self.name}')"
         sections = ["", header]
         pargsStr = self._pargsRepr()
@@ -247,20 +289,44 @@ class Instr:
         sections.append(self.body)
         return "\n".join(sections)
 
-    def register(self, session) -> Instr:
+    def register(self, renderer) -> Instr:
         """
-        Register this Instr with a Session. This is the
+        Register this Instr with a Session or an offline renderer. This is the
         same as session.registerInstr(csoundinstr)
 
         Args:
-            session (str|Session): the name of the Session or the Session itself
+            renderer: the name of a Session as str, the Session itself or
+                an offline Renderer
 
         Returns:
             self. This enables a declaration like: ``instr = Instr(...).register(session)``
+
+        Example
+        =======
+
+        .. code::
+
+            # Create an instrument and register it at a Session and
+            # at an offline Renderer
+            from csoundengine import *
+            session = Engine().session()
+            renderer = Renderer(sr=44100)
+
+            synth = Instr('synth', r'''
+                |kmidi=60|
+                outch 1, oscili:a(0.1, mtof:k(kmidi))
+            ''')
+
+            synth.register(session)
+            synth.register(renderer)
         """
-        from . import engine
-        s = session if isinstance(session, engine.Session) else engine.getSession(session)
-        s.registerInstr(self)
+        if isinstance(renderer, str):
+            from .session import getSession
+            session = getSession(renderer)
+            assert session is not None
+            session.registerInstr(self)
+        else:
+            renderer.registerInstr(self)
         return self
 
     def pargIndex(self, parg: U[int, str]) -> int:
@@ -445,7 +511,7 @@ class Instr:
         """
         return self._tableDefaultValues is not None and len(self._tableDefaultValues)>0
 
-    def overrideTable(self, d: Dict[str, float], **kws) -> List[float]:
+    def overrideTable(self, d: Dict[str, float]=None, **kws) -> List[float]:
         """
         Overrides default values in the exchange table
         Returns the initial values
@@ -496,7 +562,8 @@ def _checkInstr(instr: str) -> str:
     return errmsg
 
 
-def _parse_inline_args(body: U[str, list[str]]) -> tuple[str, Opt[dict[str, float]], str]:
+def _parse_inline_args(body: U[str, list[str]]
+                       ) -> tuple[str, Opt[dict[str, float]], str]:
     """
     Parse an instr body with a possible args declaration (see below).
 
@@ -504,12 +571,14 @@ def _parse_inline_args(body: U[str, list[str]]) -> tuple[str, Opt[dict[str, floa
         body: the body of the instrument as a string or as a list of lines
 
     Returns:
-        a tuple (delimiters, fields, body without fields declaration), with:
+        a tuple (delimiters, fields, body without fields declaration)
+
+        Where:
 
         * delimiters: the string "||" or "{}", identifying the kind of inline declaration
         * fields: a dictionary mapping field name to default value
         * body without declaration: the body of the instrument, as a string, without
-            the line declaring fields.
+          the line declaring fields.
 
         In the case that the body has no inline declaration delimiters will be an empty
         string
@@ -541,7 +610,10 @@ def _parse_inline_args(body: U[str, list[str]]) -> tuple[str, Opt[dict[str, floa
     lines = body if isinstance(body, list) else body.splitlines()
     delimiters, linenum = _detect_inline_args(lines)
     if not delimiters:
+        if isinstance(body, list):
+            body = "\n".join(body)
         return "", None, body
+    assert linenum is not None
     pfields = {}
     line2 = lines[linenum].strip()
     parts = line2[1:-1].split(",")
@@ -555,11 +627,10 @@ def _parse_inline_args(body: U[str, list[str]]) -> tuple[str, Opt[dict[str, floa
     return delimiters, pfields, body2
 
 def _tabledefGenerateCode(tabledef: dict) -> str:
-    lines = []
+    lines: List[str] = []
     idx = 0
     maxidx = len(tabledef)
-    _ = lines.append
-    _(f"""
+    lines.append(f"""
     ; --- start generated table code
     i_params = p4
     if ftexists(i_params) == 0 then
@@ -571,9 +642,9 @@ def _tabledefGenerateCode(tabledef: dict) -> str:
     endif
     """)
     for key, value in tabledef.items():
-        _(f"k{key} tab {idx}, i_params")
+        lines.append(f"k{key} tab {idx}, i_params")
         idx += 1
-    _("; --- end generated table code\n")
+    lines.append("; --- end generated table code\n")
     return textlib.joinPreservingIndentation(lines)
 
 def _pfieldsMergeDeclaration(args: Dict[str, float], body: str, startidx=4
@@ -623,6 +694,7 @@ def _pfieldsMergeDeclaration(args: Dict[str, float], body: str, startidx=4
         if body_n:
             pfields[idx] = (body_n, 0.)
         else:
+            assert args_n is not None
             pfields[idx] = (args_n, args[args_n])
     return pfields
 
